from datetime import datetime, timedelta

import openpyxl


def create_excel_result(worksheet, r, callback, config, config_knn, evaluate_metrics, training_duration_model):
    # Spaltenbreite setzen
    worksheet.column_dimensions["A"].width = 10
    worksheet.column_dimensions["B"].width = 20
    worksheet.column_dimensions["C"].width = 20
    worksheet.column_dimensions["D"].width = 20
    worksheet.column_dimensions["E"].width = 20
    worksheet.column_dimensions["F"].width = 20
    worksheet.column_dimensions["G"].width = 20
    worksheet.column_dimensions["H"].width = 20
    worksheet.column_dimensions["I"].width = 20
    worksheet.column_dimensions["J"].width = 12

    # Basis Configurationen beschreiben
    worksheet.cell(row=r, column=2).value = 'Optimierungsverfahren'
    worksheet.cell(row=r, column=3).value = 'Aktivierungsfunktion 128'
    worksheet.cell(row=r, column=4).value = 'Aktivierungsfunktion 2'
    worksheet.cell(row=r, column=5).value = 'Evaluationsmarkt'
    r = r + 1
    worksheet.cell(row=r, column=2).value = config_knn.optimization_method.name
    worksheet.cell(row=r, column=3).value = config_knn.activation_function_128_units.name
    worksheet.cell(row=r, column=4).value = config_knn.activation_function_2_units.name
    worksheet.cell(row=r, column=5).value = config_knn.excluded_folder.name
    r = r + 2

    # Schreiben Sie die Überschriften in die erste Zeile
    worksheet.cell(row=r, column=1).value = 'epoch'
    worksheet.cell(row=r, column=2).value = 'training_loss'
    worksheet.cell(row=r, column=3).value = 'training_accuracy'
    worksheet.cell(row=r, column=4).value = 'training_recall_oos'
    worksheet.cell(row=r, column=5).value = 'training_recall_not_oos'
    worksheet.cell(row=r, column=6).value = 'test_loss'
    worksheet.cell(row=r, column=7).value = 'test_accuracy'
    worksheet.cell(row=r, column=8).value = 'test_recall_oos'
    worksheet.cell(row=r, column=9).value = 'test_recall_not_oos'
    worksheet.cell(row=r, column=10).value = 'duration'
    r = r + 1

    # Schreiben Sie die Ergebnisse der Epochen in die Arbeitsmappe
    for i, (loss, accuracy, recall_oos, recall_not_oos,
            val_loss, val_accuracy, val_recall_oos, val_recall_not_oos, duration) in enumerate(
        zip(callback.epoch_training_losses, callback.epoch_training_accuracies,
            callback.epoch_training_recall_oos, callback.epoch_training_recall_not_oos,
            callback.epoch_test_losses, callback.epoch_test_accuracies, callback.epoch_test_recall_oos,
            callback.epoch_test_recall_not_oos, callback.epoch_durations)):
        worksheet.cell(row=r + i, column=1).value = i + 1
        worksheet.cell(row=r + i, column=2).value = loss
        worksheet.cell(row=r + i, column=3).value = accuracy
        worksheet.cell(row=r + i, column=4).value = recall_oos
        worksheet.cell(row=r + i, column=5).value = recall_not_oos
        worksheet.cell(row=r + i, column=6).value = val_loss
        worksheet.cell(row=r + i, column=7).value = val_accuracy
        worksheet.cell(row=r + i, column=8).value = val_recall_oos
        worksheet.cell(row=r + i, column=9).value = val_recall_not_oos
        worksheet.cell(row=r + i, column=10).value = duration
    r = r + config["knn"]["epochs"] + 1
    worksheet.cell(row=r, column=2).value = 'Trainingsdauer'
    worksheet.cell(row=r, column=3).value = training_duration_model
    r = r + 3
    worksheet.cell(row=r, column=1).value = 'Evaluation'
    worksheet.cell(row=r, column=2).value = 'loss'
    worksheet.cell(row=r, column=3).value = 'accuracy'
    worksheet.cell(row=r, column=4).value = 'recall_oos'
    worksheet.cell(row=r, column=5).value = 'recall_not_oos'
    r = r + 1
    worksheet.cell(row=r, column=1).value = config_knn.excluded_folder.name
    worksheet.cell(row=r, column=2).value = evaluate_metrics[0]
    worksheet.cell(row=r, column=3).value = evaluate_metrics[1]
    worksheet.cell(row=r, column=4).value = evaluate_metrics[3]
    worksheet.cell(row=r, column=5).value = evaluate_metrics[2]
    r = r + 4
    return r


# TODO: Abschluss Statistik nach Cross-Validation von einer Konfiguration: Durchschnitt Accuracy, Recall 1, Recall 2, Loss, Duration für Evaluation


def average_evaluate_cross_validation(worksheet, r, model_evaluate_metrics, training_duration_models):
    total_information = sum_values_into_one_array(model_evaluate_metrics, training_duration_models)
    worksheet.cell(row=r, column=2).value = 'AVERAGE'
    worksheet.cell(row=r, column=3).value = 'EVALUATE'
    worksheet.cell(row=r, column=4).value = 'METRICS'
    r = r + 1
    worksheet.cell(row=r, column=1).value = 'Models'
    worksheet.cell(row=r, column=2).value = 'loss'
    worksheet.cell(row=r, column=3).value = 'accuracy'
    worksheet.cell(row=r, column=4).value = 'recall_oos'
    worksheet.cell(row=r, column=5).value = 'recall_not_oos'
    worksheet.cell(row=r, column=6).value = 'duration'
    r = r + 1
    worksheet.cell(row=r, column=1).value = len(training_duration_models)
    worksheet.cell(row=r, column=2).value = total_information[0]
    worksheet.cell(row=r, column=3).value = total_information[1]
    worksheet.cell(row=r, column=4).value = total_information[3]
    worksheet.cell(row=r, column=5).value = total_information[2]
    # Durchschnittswert wieder in Datetime-Objekt umwandeln
    average_datetime = format(total_information[4])
    worksheet.cell(row=r, column=6).value = average_datetime


def sum_values_into_one_array(model_evaluate_metrics, training_duration_models):
    # Initialisiere Summen-Array mit Nullen
    sum_array = [0, 0, 0, 0, 0]
    num_arrays = len(model_evaluate_metrics)

    # Iteriere durch das äußere Array
    for evaluate_metric in model_evaluate_metrics:
        # Füge die Werte des inneren Arrays zum Summen-Array hinzu
        for i in range(len(evaluate_metric)):
            sum_array[i] += evaluate_metric[i]

    # Berechne den Durchschnitt, indem das Summen-Array durch die Anzahl der inneren Arrays geteilt wird
    avg_array = [value / num_arrays for value in sum_array]

    # Jetzt durchschnittsdauer des Modelltrainings berechnen
    add_times = timedelta()
    for training_time in training_duration_models:
        add_times += training_time
    # Durchschnitt berechnen
    average_duration = add_times / len(training_duration_models)
    # Ergebnis dem avg_array hinzufügen
    avg_array[4] = average_duration
    return avg_array


def get_excel_workbook():
    # Öffnen Sie eine neue Excel-Arbeitsmappe
    workbook = openpyxl.Workbook()
    return workbook


def get_excel_worksheet(workbook):
    # Fügen Sie ein neues Arbeitsblatt hinzu
    worksheet = workbook.active
    return worksheet


def save_excel(workbook, config, config_knn):
    now = datetime.now()
    date_time = now.strftime("%d_%m_%y__%H_%M")
    file_name = config_knn.optimization_method.name + "_" + \
                config_knn.activation_function_2_units.name + "_" + \
                config_knn.activation_function_128_units.name + "_" + date_time + ".xlsx"
    # Speichern Sie die Arbeitsmappe
    workbook.save(config["result"]["excel_path"] + file_name)
