import tensorflow as tf
import time


class CustomCallback(tf.keras.callbacks.Callback):
    def __init__(self):
        super().__init__()
        self.epoch_start_time = None
        self.epoch_training_losses = None
        self.epoch_training_accuracies = None
        self.epoch_training_recall_oos = None
        self.epoch_training_recall_not_oos = None
        self.epoch_test_losses = None
        self.epoch_test_accuracies = None
        self.epoch_test_recall_oos = None
        self.epoch_test_recall_not_oos = None
        self.epoch_durations = None

    def on_train_begin(self, logs=None):
        self.epoch_training_losses = []
        self.epoch_training_accuracies = []
        self.epoch_training_recall_oos = []
        self.epoch_training_recall_not_oos = []
        self.epoch_test_losses = []
        self.epoch_test_accuracies = []
        self.epoch_test_recall_oos = []
        self.epoch_test_recall_not_oos = []
        self.epoch_durations = []

    def on_epoch_begin(self, epoch, logs=None):
        self.epoch_start_time = time.time()

    def on_epoch_end(self, epoch, logs=None):
        if logs is None:
            logs = {}
        self.epoch_training_losses.append(logs.get('loss'))
        self.epoch_training_accuracies.append(logs.get('accuracy'))
        self.epoch_training_recall_oos.append(logs.get('recall_oos'))
        self.epoch_training_recall_not_oos.append(logs.get('recall_not_oos'))
        self.epoch_test_losses.append(logs.get('val_loss'))
        self.epoch_test_accuracies.append(logs.get('val_accuracy'))
        self.epoch_test_recall_oos.append(logs.get('val_recall_oos'))
        self.epoch_test_recall_not_oos.append(logs.get('val_recall_not_oos'))
        self.epoch_durations.append(time.time() - self.epoch_start_time)
